@@ -10,7 +10,7 @@ SubATA.Damped <- function(train_set, pb, qb, model.Type, accuracy.Type, level.fi
                                    , as.integer(ifelse(pb=="opt", -1, pb))
                                    , as.integer(ifelse(qb=="opt", -1, qb))
                                    , as.integer(switch(model.Type,"B"=0,"A"=1,"M"=2))
-                                   , as.integer(switch(accuracy.Type,"MAE"=1,"MdAE"=2,"MSE"=3,"MdSE"=4,"MPE"=5,"MdPE"=6,"MAPE"=7,"MdAPE"=8,"sMAPE"=9,"sMdAPE"=10,"RMSE"=11,"MASE"=12,"OWA"=13,"AMSE"=14,"lik"=15,"sigma"=16))
+                                   , as.integer(switch(accuracy.Type,"MAE"=1,"MdAE"=2,"MSE"=3,"MdSE"=4,"MPE"=5,"MdPE"=6,"MAPE"=7,"MdAPE"=8,"sMAPE"=9,"sMdAPE"=10,"RMSE"=11,"MASE"=12,"OWA"=13,"AMSE"=14,"lik"=15,"sigma"=16,"GAMSE"=17))
                                    , as.integer(ifelse(level.fix, 1, 0))
                                    , as.integer(ifelse(trend.fix, 1, 0))
                                    , as.integer(ifelse(trend.Search, 1, 0))
@@ -28,7 +28,7 @@ SubATA.Damped <- function(train_set, pb, qb, model.Type, accuracy.Type, level.fi
                                    , as.integer(ifelse(pb=="opt", -1, pb))
                                    , as.integer(ifelse(qb=="opt", -1, qb))
                                    , as.integer(switch(model.Type,"B"=0,"A"=1,"M"=2))
-                                   , as.integer(switch(accuracy.Type,"MAE"=1,"MdAE"=2,"MSE"=3,"MdSE"=4,"MPE"=5,"MdPE"=6,"MAPE"=7,"MdAPE"=8,"sMAPE"=9,"sMdAPE"=10,"RMSE"=11,"MASE"=12,"OWA"=13,"AMSE"=14,"lik"=15,"sigma"=16))
+                                   , as.integer(switch(accuracy.Type,"MAE"=1,"MdAE"=2,"MSE"=3,"MdSE"=4,"MPE"=5,"MdPE"=6,"MAPE"=7,"MdAPE"=8,"sMAPE"=9,"sMdAPE"=10,"RMSE"=11,"MASE"=12,"OWA"=13,"AMSE"=14,"lik"=15,"sigma"=16,"GAMSE"=17))
                                    , as.integer(ifelse(level.fix, 1, 0))
                                    , as.integer(ifelse(trend.fix, 1, 0))
                                    , as.integer(ifelse(trend.Search, 1, 0))
@@ -47,7 +47,7 @@ SubATA.Damped <- function(train_set, pb, qb, model.Type, accuracy.Type, level.fi
                             , as.integer(ifelse(pb=="opt", -1, pb))
                             , as.integer(ifelse(qb=="opt", -1, qb))
                             , as.integer(switch(model.Type,"B"=0,"A"=1,"M"=2))
-                            , as.integer(switch(accuracy.Type,"MAE"=1,"MdAE"=2,"MSE"=3,"MdSE"=4,"MPE"=5,"MdPE"=6,"MAPE"=7,"MdAPE"=8,"sMAPE"=9,"sMdAPE"=10,"RMSE"=11,"MASE"=12,"OWA"=13,"AMSE"=14,"lik"=15,"sigma"=16))
+                            , as.integer(switch(accuracy.Type,"MAE"=1,"MdAE"=2,"MSE"=3,"MdSE"=4,"MPE"=5,"MdPE"=6,"MAPE"=7,"MdAPE"=8,"sMAPE"=9,"sMdAPE"=10,"RMSE"=11,"MASE"=12,"OWA"=13,"AMSE"=14,"lik"=15,"sigma"=16,"GAMSE"=17))
                             , as.integer(ifelse(level.fix, 1, 0))
                             , as.integer(ifelse(trend.fix, 1, 0))
                             , as.integer(ifelse(trend.Search, 1, 0))
@@ -62,7 +62,7 @@ SubATA.Damped <- function(train_set, pb, qb, model.Type, accuracy.Type, level.fi
                             , as.integer(nmse))
   }
   ifelse(Holdout==TRUE & Adjusted_P==TRUE, new_pk <- round((output[1] * length(main_set))/ length(train_set)), new_pk <- output[1])
-  ATA.last <- ATA.Core(main_set, pk = new_pk, qk = output[2], phik = output[3], mdlType = ifelse(output[4]==1,"A","M"), initialLevel = initialLevel, initialTrend = initialTrend, nmse)
+  ATA.last <- ATA.Core(main_set, pk = new_pk, qk = output[2], phik = output[3], mdlType = ifelse(output[4]==1,"A","M"), initialLevel = initialLevel, initialTrend = initialTrend)
   ATA.last$holdout <- Holdout
   ATA.last$holdin <- Holdin
   if(Holdout==TRUE){
